#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

require "html.pl";
require "db.pl";
require "dbops.pl";
require "url.pl";
require "common.pl";
require "lang.pl";
require "urllist.pl";


&main();
exit(0);


sub main
{
	&HTML::print_content();
	&HTML::print_header();

	my $chan = &HTML::get("chan", 1);
	my $seek = &HTML::get("seek", -1);
	my $n = &HTML::get("n", -1);

	if ($seek == -1) {
		&summary($chan);
	} else {
		&url_hist($chan, $n, $seek);
	}

	&HTML::print_footer(&DB::process_time());
}


# 0: chan
sub summary
{
	my ($id) = @_;

	my $chan = &DBOps::get_chan_name($id);
	(defined($chan)) || &HTML::bad_param("bad chan");

	printf "<h1>" . &Lang::get("chan-summary") . "</h1>\n", $chan;

	&combined_top($id, $chan, 5);
	&urls($id, $chan, 10);
}


# 0: chan
# 1: n
# 2: seek
sub url_hist
{
	my ($id, $n, $seek) = @_;

	my $chan = &DBOps::get_chan_name($id);
	(defined($chan)) || &HTML::bad_param("bad chan");

	printf "<h1>" . &Lang::get("chan-hist-for") . "</h1>\n", $chan;

	&urls($id, $chan, $n, $seek);
}


# 0: chan id
# 1: chan name
# 2: n
sub combined_top
{
	my ($id, $name, $n) = @_;

	my @fame = &get_top($id, $n, 1);
	my @shame = &get_top($id, $n, 0);
	my @paster = &get_pasters($id, $n);

	($#fame == -1 && $#shame == -1 && $#paster == -1) && return;

	my $l_rank = &Lang::get("col-rank");
	my $l_nick = &Lang::get("col-nick");
	my $l_points = &Lang::get("col-points");
	my $l_urls = &Lang::get("col-urls");
	my $l_point_p = &Lang::get("col-points-ratio");
	my $l_fame = &Lang::get("col-fame");
	my $l_shame = &Lang::get("col-shame");
	my $l_busy = &Lang::get("col-busy");

	print <<EOF;
<table class="wall">
	<colgroup span="2" class="fame" />
	<colgroup class="shared" />
	<colgroup span="2" class="shame" />
	<colgroup class="shared" />
	<colgroup span="4" class="paster" />

	<tr>
		<th colspan="2">$l_fame</th>
		<th class="srank"></th>
		<th colspan="2">$l_shame</th>
		<th class="srank"></th>
		<th colspan="4">$l_busy</th>

	</tr>
	<tr class="ul">
		<th>$l_nick</th>
		<th>$l_points</th>
		<th class="srank">$l_rank</th>
		<th>$l_nick</th>
		<th>$l_points</th>
		<th class="srank">$l_rank</th>
		<th>$l_nick</th>
		<th>$l_urls</th>
		<th>$l_points</th>
		<th>$l_point_p</th>
	</tr>
EOF

	my $rn = &Common::max(&Common::max($#fame, $#shame), $#paster) + 1;
	for (my $i = 0; $i < $rn; $i++) {
		my $class = $i % 2 == 1 ? " class=\"zebra\"" : "";
		print "\t<tr$class>\n";

		&print_fame_shame($fame[$i]);
		print "\t\t<td class=\"srank\">#" . ($i + 1) . "</td>\n";
		&print_fame_shame($shame[$i]);
		print "\t\t<td class=\"srank\">#" . ($i + 1) . "</td>\n";
		&print_paster($paster[$i]);

		print "\t</tr>\n";
	}

	print "</table>\n";
}


# 0: csv
sub print_fame_shame
{
	if (! defined($_[0])) {
		print "\t\t<td colspan=\"2\"></td>\n";
		return;
	}
	
	my ($t_id, $t_score, $t_nick) = split(/,/, $_[0], 3);

	my $a_user = &URL::user($t_id);
	print "\t\t<td><a href=\"$a_user\">$t_nick</a></td>\n";
	print "\t\t<td class=\"center\">$t_score</td>\n";
}


# 0: csv
sub print_paster
{
	my ($line) = @_;
	if (! defined($line)) {
		print "\t\t<td colspan=\"4\"></td>\n";
		return;
	}

	my ($id, $urls, $score, $nick) = split(/,/, $line, 4);

	my $a_user = &URL::user($id);
	my $hit_r = sprintf("%.1f", $score * 100 / $urls);

	print <<EOF;
		<td><a href="$a_user">$nick</a></td>
		<td class="center">$urls</td>
		<td class="center">$score</td>
		<td class="center">$hit_r%</td>
EOF
}


# 0: chan id
# 1: n
# 2: fame
# r: csv list (id,score,nick)
sub get_top
{
	my ($chan_id, $n, $fame) = @_;
	my ($query, $result);

	my $gcol = $fame == 1 ? "fame" : "shame";
	my $bcol = $fame == 1 ? "shame" : "fame";
	$query = "SELECT nicks.id, nicks.nick, scoreboard.$gcol " .
			"FROM nicks, scoreboard " .
			"WHERE scoreboard.chan = $chan_id " .
				"AND scoreboard.nick = nicks.id " .
			"ORDER BY scoreboard.$gcol DESC, " .
				"scoreboard.$bcol ASC " .
			"LIMIT $n";
	$result = &DB::query($query);
	if ($result->ntuples == 0) {
		return;
	}

	my @ret;
	while (my ($user_id, $user_nick, $user_score) = $result->fetchrow) {
		my $r = "$user_id,$user_score,$user_nick";
		push(@ret, $r);
	}
	return @ret;
}


# 0: chan id
# 1: chan name
# 2: n
# 3: seek
sub urls
{
	my ($chan_id, $chan_name, $n, $seek) = @_;
	my @urls;
	my @images;
	my $has_more;

	$n = 25 if ($n > 25);

	# get one extra so that we know if there's more
	@urls = &get_urls($chan_id, $n + 1, $seek);
	($#urls == -1) && return;
	if ($#urls == $n) {
		$has_more = 1;
		pop @urls;
	} else {
		$has_more = 0;
	}

	if (! defined($seek)) {
		printf "<h2>" . &Lang::get("chan-last-n-urls") . "</h2>\n", $n;
	} else {
		my ($first) = $urls[0] =~ /^(\d+),/;
		my ($last) = $urls[$#urls] =~ /^(\d+),/;
		printf "<h2>" . &Lang::get("chan-urls-range") . "</h2>\n",
				$first, $last;
	}

	@images = &URLList::print(defined($seek) ? 1 : 0, @urls);

	my $l;
	$l = &Lang::get("link-chanstat");
	print "<p>" . &Lang::get("link-go-to") .
			"<a href=\"" . &URL::chanstat($chan_id) . "\">" .
			"\u$l</a>,\n";
	if (! defined($seek)) {
		my $a_chan = &URL::chan($chan_id, 25, $n);
		my $l = &Lang::get("link-older-urls");
		print "\t<a href=\"$a_chan\">$l</a>\n";
	} else {
		my ($a_fresh, $a_new, $a_old);
		if ($seek > 0) {
			my $tseek = $seek - $n;
			$tseek = 0 if ($tseek < 0);
			my $a_chan = &URL::chan($chan_id, $n, $tseek);
			$a_new = ",\n\t<a href=\"$a_chan\">" .
					&Lang::get("link-newer") . "</a>";
		} else {
			$a_new = "";
		}
		if ($has_more == 1) {
			my $tseek = $seek + $n;
			my $a_chan = &URL::chan($chan_id, $n, $tseek);
			$a_old = ",\n\t<a href=\"$a_chan\">" .
					&Lang::get("link-older") . "</a>";
		} else {
			$a_old = "";
		}
		$a_fresh = &URL::chan($chan_id);
		$l = &Lang::get("link-latest-urls");
		print "\t<a href=\"$a_fresh\">$l</a>$a_new$a_old\n";
	}
	print "</p>\n";

	&Common::link_images(@images);
}


# 0: chan id
# 1: n
# 2: seek
# r: csv (url_id,url_time,url_count,nick_id,nick_name,url)
sub get_urls
{
	my ($chan_id, $n, $seek) = @_;
	my ($query, $result);
	my @ret;

	$query = "SELECT urls.id, urls.url, " .
				&DBOps::get_date_fmt("urls.time") . ", " .
				"urls.count, nicks.nick, nicks.id " .
			"FROM urls, nicks " .
			"WHERE urls.chan = $chan_id AND urls.nick = nicks.id " .
			"ORDER BY urls.id DESC " .
			"LIMIT $n";
	$query .= " OFFSET $seek" if (defined($seek));

	$result = &DB::query($query);
	($result->ntuples == 0) && return ();

	while (my ($url_id, $url_url, $url_time, $url_count,
				$nick_name, $nick_id) =
			$result->fetchrow) {
		$nick_name =~ s/,/./g;
		push(@ret, "$url_id,$url_time,$url_count," .
				"$nick_id,$nick_name,$url_url");
	}
	return @ret;
}


# 0: chan id
# 1: n
# r: csv (nick_id, urls, fame, nick_name)
sub get_pasters
{
	my ($chan_id, $n) = @_;
	my ($query, $result);
	my @ret;

	my $subq = "SELECT nicks.id, nicks.nick, SUM(urls.count) AS sum " .
			"FROM nicks, urls " .
			"WHERE nicks.id = urls.nick AND urls.chan = $chan_id " .
			"GROUP BY nicks.id, nicks.nick";
	$query = "SELECT sum.id, sum.nick, sum.sum, score.fame " .
			"FROM ($subq) AS sum, scoreboard AS score " .
			"WHERE sum.id = score.nick " .
			"ORDER BY sum DESC LIMIT $n";
	$result = &DB::query($query);
	if ($result->ntuples == 0) {
		return;
	}

	while (my ($nick_id, $nick_name, $url_count, $fame) =
			$result->fetchrow) {
		push(@ret, "$nick_id,$url_count,$fame,$nick_name");
	}

	return @ret;
}
